<?php

namespace FiloBlu\Refilo\Model\Indexer\Processor\Blocks\Rendered;

use Magento\CatalogWidget\Model\RuleFactory;
use Magento\Framework\Data\Wysiwyg\Normalizer as FilterNormalized;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Filter\Template as FilterTemplate;
use Magento\Framework\Serialize\Serializer\Json as SerializerJson;
use Magento\Rule\Model\Condition\AbstractCondition;
use Magento\Rule\Model\Condition\Combine;

/**
 * Class Conditions
 * @package FiloBlu\Refilo\Model\Indexer\Processor\Blocks\Rendered
 */
class Conditions
{
    /**
     * @var SerializerJson
     */
    private $serializerJson;
    /**
     * @var RuleFactory
     */
    private $rule;
    /**
     * @var FilterTemplate
     */
    private $filterTemplate;
    /**
     * @var FilterNormalized
     */
    private $filterNormalizer;

    /**
     * @var array
     */
    protected $_conditionOperatorMap = [
        '=='    => 'eq',
        '!='    => 'neq',
        '>='    => 'gte',
        '&gt;=' => 'gte',
        '>'     => 'gt',
        '&gt;'  => 'gt',
        '<='    => 'lte',
        '&lt;=' => 'lte',
        '<'     => 'lt',
        '&lt;'  => 'lt',
        '{}'    => 'inq',
        '!{}'   => 'nin',
        '()'    => 'inq',
        '!()'   => 'nin',
    ];

    /**
     * @var array
     */
    private $stringConditionOperatorMap = [
        '{}' => 'like',
        '!{}' => 'nlike',
    ];

    /**
     * Catalog constructor.
     * @param FilterTemplate $filterTemplate
     * @param SerializerJson $serializerJson
     * @param RuleFactory $ruleFactory
     * @param FilterNormalized $filterNormalizer
     */
    public function __construct(
        FilterTemplate $filterTemplate,
        SerializerJson $serializerJson,
        RuleFactory $ruleFactory,
        FilterNormalized $filterNormalizer
    ) {
        $this->serializerJson = $serializerJson;
        $this->rule = $ruleFactory->create();
        $this->filterTemplate = $filterTemplate;
        $this->filterNormalizer = $filterNormalizer;
    }

    /**
     * @param string $conditions
     * @return array
     * @throws LocalizedException
     */
    public function process($conditions)
    {
        $block_decoded = $this->serializerJson->unserialize(
            $this->filterNormalizer->restoreReservedCharacters($conditions)
        );
        $this->rule->loadPost(['conditions'=>$block_decoded]);
        $rules = $this->_getMappedCombination($this->rule->getConditions());
        return json_decode(json_encode($rules));
    }

    /**
     * Get mapped sql combination.
     *
     * @param Combine $combine
     * @param string $value
     * @param bool $isDefaultStoreUsed
     * @return array
     * @throws LocalizedException
     * @SuppressWarnings(PHPMD.NPathComplexity)
     */
    protected function _getMappedCombination(
        Combine $combine,
        &$argument=''
    ) {
//        $out = (!empty($value) ? $value : '');
        $value = ($combine->getValue() ? '' : ' NOT ');
        $getAggregator = $combine->getAggregator();
        $conditions = $combine->getConditions();
//        $rules = new \stdClass();
        foreach ($conditions as $key => $condition) {
            /** @var $condition AbstractCondition|Combine */
            $con = ($getAggregator == 'any' ? "or" : "and");
//            $con = (isset($conditions[$key+1]) ? $con : '');
            if ($condition instanceof Combine) {
                $rules[$con][$argument] = $this->_getMappedCombination($condition,$argument);
            } else {
                $rules[$con][$argument]  = $this->_getMappedCondition($condition,$argument);
            }
        }

        return (object)$rules;
    }

    /**
     * Returns sql expression based on rule condition.
     *
     * @param AbstractCondition $condition
     * @return array
     * @throws LocalizedException
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    protected function _getMappedCondition(
        AbstractCondition $condition,
        &$argument
    ) {
        $argument = $condition->getAttribute();

        // If rule hasn't valid argument - prevent incorrect rule behavior.
        if (empty($argument)) {
            return $this->_expressionFactory->create(['expression' => '1 = -1']);
        } elseif (preg_match('/[^a-z0-9\-_\.\`]/i', $argument) > 0) {
            throw new LocalizedException(__('Invalid field'));
        }

        $conditionOperator = $condition->getOperatorForValidate();

        if (!isset($this->_conditionOperatorMap[$conditionOperator])) {
            throw new LocalizedException(__('Unknown condition operator'));
        }

        if ($condition->getInputType() === 'string'
            && in_array($conditionOperator, array_keys($this->stringConditionOperatorMap), true)
        ) {
            $operator = str_replace(
                ':field',
                $argument,
                $this->stringConditionOperatorMap[$conditionOperator]
            );
            $bindValue = $condition->getBindArgumentValue();
            $expression[$argument] = [$operator=>$bindValue];
        } else {
            $operator = str_replace(
                ':field',
                $argument,
                $this->_conditionOperatorMap[$conditionOperator]
            );
            $bindValue = $condition->getBindArgumentValue();
            $expression = [$operator=>$bindValue];
        }

        if (is_array($bindValue) && \in_array($conditionOperator, ['()', '{}'], true)) {
            foreach ($bindValue as $item) {
                $expression .= $this->_connection->quoteInto(
                    " OR (FIND_IN_SET (?, {$$argument}) > 0)",
                    $item
                );
            }
        }
        return $expression;
    }
}
//            $data->field = "id";
//            $data->operator = "=";
//            $data->value = $id;
//            $condition_serialized = $this->serializerJson->serialize($data);
