<?php

namespace FiloBlu\Refilo\Remote\Connector;

use Elasticsearch\Client;
use Elasticsearch\ClientBuilder;
use Traversable;

/**
 * Class ElasticSearchConnector
 * @package FiloBlu\Refilo\Remote\Connector
 */
class ElasticSearch extends AbstractConnector
{
    /**
     * @var Client
     */
    protected $handler;

    /**
     * @return mixed|void
     */
    public function disconnect(): ConnectorInterface
    {
        if (!$this->handler) {
            return $this;
        }

        $this->handler = null;

        return $this;
    }

    /**
     * @param Traversable $entities
     * @param string $collection
     * @return mixed|void
     */
    public function create(Traversable $entities, string $collection)
    {
        $this->createIndex($collection);

        foreach ($entities as $entity) {
            $this->handler->index([
                'index' => $collection,
                'type'  => '_doc',
                'id'    => $entity->getId(),
                'body'  => $entity->toArray()
            ]);
        }
    }

    /**
     * @param $index
     * @return $this
     */
    public function createIndex($index)
    {
        $client = $this->getHandler();
        $indices = $client->indices()->get(['index' => $index, 'ignore_unavailable' => true]);

        if (array_key_exists($index, $indices)) {
            return $this;
        }

        $indexes['autosuggests']['type'] = 'completion';
        $indexes['name']['type'] = 'text';
        $indexes['name']['copy_to'] = 'autosuggests';
        $indexes['name']['fields']['keyword']['type'] = 'keyword';
        $indexes['sku']['type'] = 'keyword';
        $indexes['id']['type'] = 'long';

        $mapping = [
            'enabled'           => true,
            '_source'           => ['enabled' => true],
            'dynamic_templates' => [
                [
                    'attributes' => [
                        'match_mapping_type' => 'string',
                        'path_match'         => 'attributes.*',
                        'mapping'            => [
                            'type'         => 'keyword',
                            'ignore_above' => 256
                        ]
                    ],
                ],
                [
                    'filters' => [
                        'match_mapping_type' => 'string',
                        'path_match'         => 'filters.*',
                        'mapping'            => [
                            'type'         => 'keyword',
                            'ignore_above' => 256
                        ]
                    ],
                ],
                [
                    'sortables' => [
                        'match_mapping_type' => 'string',
                        'path_match'         => 'sortables.*',
                        'mapping'            => [
                            'type'         => 'keyword',
                            'ignore_above' => 256
                        ]
                    ],
                ]
            ],
            'properties'        => $indexes
        ];

        $params = [
            'index' => $index,
            'body'  => [
                'settings' => [
                    'index.priority'                      => '1',
                    'index.query.default_field'           => ['*'],
                    'index.refresh_interval'              => '1s',
                    'index.write.wait_for_active_shards'  => '1',
                    'index.mapping.total_fields.limit'    => '1000',
                    'index.blocks.read_only_allow_delete' => 'false',
                    'index.number_of_replicas'            => '0'
                ],
                // sulla 7.
                //   'mappings' =>  $mapping
                // sulla 6.0
                'mappings' => ['_doc' => $mapping]
            ]
        ];

        // $deleteParams = [
        //    'index' => $index_prefix . '_' . $store
        // ];
        //$this->_connection->indices()->delete($deleteParams);
        $client->indices()->create($params);

        return $this;
    }

    protected function getHandler()
    {

        $this->connect();

        return $this->handler;
    }

    /**
     * @return ConnectorInterface
     */
    public function connect(): ConnectorInterface
    {
        if (!$this->handler) {
            $this->handler = ClientBuilder::create()
                ->setRetries(2)
                ->setHosts($this->getHosts())
                ->build();
        }

        return $this;
    }

    /**
     * @return array
     */
    protected function getHosts(): array
    {
        $hosts = $this->getConfiguration()->getData(self::CONFIGURATION_HOST_POOL);

        $dsns = [];
        foreach ($hosts as $host) {
            $dsn = "{$host['scheme']}://";

            if ($host['username']) {
                $dsn .= "{$host['username']}:{$host['password']}@";
            }

            $dsn .= $host['host'];

            if ($host['port']) {
                $dsn .= ":{$host['port']}";
            }
            $dsns[] = $dsn;
        }

        return $dsns;
    }

    /**
     * @param array $entity
     * @param string $collection
     * @return mixed|void
     */
    public function delete(array $entity, string $collection)
    {
    }

    /**
     * @param Traversable $entities
     * @param string $collection
     * @return mixed|void
     */
    public function update(\Traversable $entities, string $collection)
    {
        $this->createIndex($collection);

        foreach ($entities as $object) {
            $responses = $this->handler->update(
                [
                    'index' => $collection,
                    'id'    => $object->getId(),
                    'type'  => '_doc',
                    'body'  => [
                        'doc' => $object->toArray()
                    ]
                ]
            );
        }


    }

    /**
     * @return array|mixed|null
     */
    protected function getPrefix()
    {
        return $this->getConfiguration()->getData('prefix');
    }
}
