<?php

namespace FiloBlu\Refilo\Remote\Entity\Provider;

use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Eav\Api\AttributeRepositoryInterface;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Store\Api\Data\StoreInterface;
use Magento\Store\Model\StoreManagerInterface;

/**
 * Class AttributeProvider
 * @package FiloBlu\Refilo\Remote\Entity\Provider
 */
class ProductAttributeProvider
{
    /**
     * TODO: attribute to exclude must be configurable
     * @var string[]
     */
    const EXCLUDED_ATTRIBUTES = [
        'allow_message',
        'allow_open_amount',
        'created_at',
        'custom_design',
        'custom_design_from',
        'custom_design_to',
        'custom_layout',
        'custom_layout_update',
        'custom_layout_update_file',
        'email_template',
        'has_options',
        'media_gallery',
        'minimal_price',
        'price',
        'msrp_display_actual_price_type',
        'old_id',
        'open_amount_max',
        'open_amount_min',
        'image',
        'image_label',
        'links_exist',
        'links_purchased_separately',
        'links_title',
        'merchant_center_category',
        'msrp',
        'open_amount_max',
        'open_amount_min',
        'options_container',
        'origin_storage',
        'package_mode',
        'package_type',
        'page_layout',
        'price_type',
        'price_view',
        'quantity_and_stock_status',
        'related_tgtr_position_behavior',
        'related_tgtr_position_limit',
        'required_options',
        'small_image',
        'small_image_label',
        'special_price',
        'status',
        'swatch_image',
        'thumbnail',
        'thumbnail_label',
        'tier_price',
        'type_of_product',
        'updated_at',
        'upsell_tgtr_position_behavior',
        'upsell_tgtr_position_limit',
        'use_config_allow_message',
        'use_config_email_template',
        'use_config_is_redeemable',
        'visibility',
        'shipment_type'
    ];
    /**
     * @var array
     */
    protected $imagePlaceholderAttributes = [];

    /** @var array */
    protected $attributes;

    /** @var array */
    protected $filterableAttributes = [];

    /** @var array */
    protected $sortableAttributes = [];
    /**
     * @var array
     */
    protected $lookupOptions = [];
    /**
     * @var SearchCriteriaBuilder
     */
    private $searchCriteriaBuilder;
    /**
     * @var AttributeRepositoryInterface
     */
    private $attributeRepository;
    /**
     * @var StoreManagerInterface
     */
    private $storeManager;

    /**
     * AttributeProvider constructor.
     * @param SearchCriteriaBuilder $searchCriteriaBuilder
     * @param AttributeRepositoryInterface $attributeRepository
     * @param StoreManagerInterface $storeManager
     */
    public function __construct(
        SearchCriteriaBuilder $searchCriteriaBuilder,
        AttributeRepositoryInterface $attributeRepository,
        StoreManagerInterface $storeManager
    ) {
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->attributeRepository = $attributeRepository;
        $this->storeManager = $storeManager;
    }

    /**
     * @param StoreInterface $store
     * @return mixed
     */
    public function getAttributes(StoreInterface $store)
    {
        $this->init();
        return $this->attributes[$store->getId()];
    }

    /**
     *
     */
    protected function init()
    {
        if ($this->attributes !== null) {
            return;
        }

        $searchCriteria = $this->searchCriteriaBuilder->create();
        $response = $this->attributeRepository->getList('catalog_product', $searchCriteria);

        foreach ($this->storeManager->getStores() as $store) {
            $storeId = $store->getId();
            $this->filterableAttributes[$storeId] = [];
            $this->sortableAttributes[$storeId] = [];
            $this->attributes[$storeId] = [];
        }

        foreach ($this->storeManager->getStores() as $store) {
            $storeId = $store->getId();
            foreach ($response->getItems() as $attribute) {
                $attributeCode = $attribute->getAttributeCode();

                if ((bool)$attribute->getIsVisibleOnFront() || (bool)$attribute->getUsedInProductListing()) {
                    $this->attributes[$storeId][$attributeCode] = [
                        'title' => $attribute->getStoreLabel($store->getId()),
                        'code'  => $attributeCode,
                        'type'  => $attribute->getFrontendInput()
                    ];

                    if ($attribute->getIsFilterable()) {
                        $this->filterableAttributes[$storeId][$attributeCode] = $this->attributes[$storeId][$attributeCode];
                    }

                    if ($attribute->getUsedForSortBy()) {
                        $this->sortableAttributes[$storeId][$attributeCode] = $this->attributes[$storeId][$attributeCode];
                    }
                }

                if ($attribute->getFrontendInput() === 'media_image') {
                    $this->imagePlaceholderAttributes[$attributeCode] = $attributeCode;
                }

                /*  if($attribute->getFrontend()->getConfigField('input') === 'boolean')
                  {

                  }*/

                if ($attribute->usesSource()) {
                    foreach ($attribute->getSource()->getAllOptions() as $option) {
                        $this->lookupOptions[$store->getId()][$attributeCode][$option['value']] = (string)$option['label'];
                    }
                }
            }
        }
    }

    /**
     * @param ProductInterface $product
     * @return array
     */
    public function getAttributesValues(ProductInterface $product)
    {
        $this->init();
        return $this->getAttributesFromProduct($product, $this->attributes, self::EXCLUDED_ATTRIBUTES);
    }

    /**
     * @param ProductInterface $product
     * @param array $attributeList
     * @param array $excludeList
     * @return array
     */
    protected function getAttributesFromProduct(ProductInterface $product, array $attributeList, array $excludeList = []): array
    {
        $storeId = $product->getStore()->getId();

        $attributes = [];

        foreach ($attributeList[$storeId] as $code => $attribute) {

            if (in_array($code, $excludeList, true)) {
                continue;
            }

            $type = $attribute['type'];
            switch ($type) {
                case 'select':
                    $attributes[$code] = [
                        'code'  => $code,
                        'title' => $attribute['title'],
                        'type'  => $type,
                        'value' => $this->lookupOptions[$storeId][$code][$product->getData($code)] ?? null,
                        'key'   => $product->getData($code)
                    ];
                    break;
                case 'multiselect':
                    // TODO: getAttributeText is slow, prefetch values
                    $attributes[$code] =
                        [
                            'code'  => $code,
                            'title' => $attribute['title'],
                            'type'  => $type,
                            'value' => $product->getAttributeText($code),
                            'key'   => $product->getData($code)
                        ];
                    break;
                default:
                    $attributes[$code] =
                        [
                            'code'  => $code,
                            'title' => $attribute['title'],
                            'type'  => $type,
                            'value' => $product->getData($code)
                        ];
                    break;
            }
        }

        /*      if (
                 isset($this->_product_attributes[$attribute->getAttributeCode()]["additional_data"]) &&
                 isset($this->_swatch_options[$this->_item->attributes->{$attribute_code}->key])
             ) {
                 $additional_object = json_decode($this->_product_attributes[$attribute->getAttributeCode()]["additional_data"]);

                 if (property_exists($additional_object, "swatch_input_type")) {
                     $this->_item->attributes->{$attribute_code}->image = '/attribute/swatch' . $this->_swatch_options[$this->_item->attributes->{$attribute_code}->key]["value"];
                     $this->_item->attributes->{$attribute_code}->type = $additional_object->swatch_input_type;
                 }
             }*/

        return $attributes;
    }

    /**
     * @param ProductInterface $product
     * @return array
     */
    public function getFilterableAttributes(ProductInterface $product): array
    {
        $this->init();
        return $this->getAttributesFromProduct($product, $this->filterableAttributes);
    }

    /**
     * @param ProductInterface $product
     * @return array
     */
    public function getSortableAttributes(ProductInterface $product)
    {
        $this->init();
        return $this->getAttributesFromProduct($product, $this->sortableAttributes);
    }

    /**
     * @param ProductInterface $product
     * @return array
     */
    public function getImagePlaceholdersAttributes(ProductInterface $product): array
    {
        $this->init();

        $placeholders = [];

        foreach ($this->imagePlaceholderAttributes as $attributeCode) {
            $value = $product->getData($attributeCode);
            if ($value === null) {
                continue;
            }
            $placeholders[$attributeCode] = "/catalog/product{$value}";
        }

        return $placeholders;
    }
}
